
import 'dart:async';
import 'dart:collection';
import 'dart:convert';

import 'package:flutter/services.dart';

class ResultEntry
{
  ResultEntry(this.poi, this.score);

  /// The string id of the poi
  final String poi;

  /// The recommendation score
  final double score;

  static ResultEntry fromJson(String str) {
    Map<String,dynamic> obj = json.decode(str);
    return ResultEntry(obj['poi'] as String, obj['score'] as double);
  }
}

class Lgcre {

  static const MethodChannel _channel =
      const MethodChannel('lgcre');

  /// Initializes LGCRE with the given [options]
  /// 
  /// Available [options] are :
  /// 
  ///     ['lgcre.licenseid']: the license id
  ///
  ///     ['lgcre.clientid']: the client id
  ///
  ///     ['lgcre.loglevel']: the log level (off, fine, warning, debug, severe, config),
  ///
  ///     ['lgcre.maxmem']: the maximum amount of memory to use (affects the speed)
  static Future<void> init(Map<String,String> options) async
  {
    return await _channel.invokeMethod<void>('init', options);
  }

  /// Resets the stored data
  static Future<void> reset() async
  {
    await _channel.invokeMethod('reset');
  }

  /// Adds [data] to the recommender engine
  ///
  /// The [data] map can contain has the following fields
  ///
  ///     ['pois'] : a list of strings identifying the pois
  ///
  ///     ['categories'] : a list of strings identifying the pois
  ///
  ///     ['links_categories'] : a list of maps with:
  ///
  ///           ['category1']: The string id of the first category to link
  ///
  ///           ['category2']: The string id of the second category to link
  ///
  ///     ['links_poi_category']: a list of maps with:
  ///
  ///           ['poi']: the string id of the poi to link
  ///
  ///           ['category']: the stdring id of the category to link
  ///
  ///     ['person']: A map with:
  ///
  ///       ['pois']: a list of string ids of pois a person like/visited
  ///
  ///       ['categories']: a list of string ids of categories the person is interested in 
  ///
  /// Passing a non valid key in the map returns an error.
  static Future<void> addData(Map<String, dynamic> data)  async
  {
    await _channel.invokeMethod('addData', data);
  }

  static Future<void> release() async
  {
    await _channel.invokeMethod('release');
  }

  /// Runs a recommendation [query]
  ///
  /// The [query] is a map with:
  ///
  ///     ['pois']: a list of maps with
  ///         ['poi']: an string with the poi identifier
  ///         ['categories']: a list of string ids with the categories of the poi
  ///
  /// The query returns a list of [ResultEntry] sorted decreasingly by the
  /// [ResultEntry] score.
  static Future<List<ResultEntry>?> run(Map<String,dynamic> query) async
  {
    final List<String>? entries = await _channel.invokeListMethod<String>('run', query);
    return entries?.map(ResultEntry.fromJson).toList() ?? <ResultEntry>[];
  }
}
